#!/usr/bin/env python
# -*- coding: latin-1 -*-

# Imports
import pygame
from pygameutils import make_bat, make_ghost
from numarray import array

# ===========================  BAT  ==========================================
class Bat:
	""" Class that represent a bat/paddle """
	def __init__(self, xy, offset, playerNumber):
		""" Init a basic bat, this is the one that the server uses """
		self.rect = pygame.Rect(xy, (20,60))
		self.leftRect = pygame.Rect(xy, (2,60))
		self.rightRect = pygame.Rect(xy, (2,60))
		self.upRect = pygame.Rect(xy, (20,2))
		self.downRect = pygame.Rect(xy, (20,2))
		self.offset = offset
		self.playerNumber = playerNumber

	def move(self, xy):
		""" Moves the ba/paddle to position xy """
		# Hard coded limits where the bats are allowed to be
		limits = [510, 582, 7, 233]
		if self.playerNumber == 2:
			limits = [18, 90, 7, 233]

		newPos = self.rect.midtop
		# Clamp into x-coordinates
		if xy[0] > limits[0] and xy[0] < limits[1]:
			newPos = xy[0], newPos[1]
		# Clamp into y-coordinates
		if	xy[1] > limits[2] and xy[1] < limits[3]:
			newPos = newPos[0], xy[1]
		self.rect.midtop = newPos[0], newPos[1] + self.offset
		self.updateRects()
		return self.rect.midtop


	def updateRects(self):
		""" Method that updates the rects that is used for collision
			detection """
		pos = self.rect.midtop
		self.leftRect.topleft = (pos[0] - 10, pos[1])
		self.rightRect.topleft = (pos[0] + 10 , pos[1])
		self.upRect.topleft = (pos[0] - 10, pos[1])
		self.downRect.topleft = (pos[0] - 10, pos[1] + 58)


# ===========================  CLIENTBAT  ====================================
class ClientBat(pygame.sprite.Sprite, Bat):
	""" This is the Bat class that is used on the clients, it initialized
		with an image and a rect that belongs to the object """
	def __init__(self, xy, screen, offset, color=(200,200,200)):
		pygame.sprite.Sprite.__init__(self)
		Bat.__init__(self, xy, offset, 1)
		self.size = (20, 60)
		self.color = color
		self.width = 2
		self.screen = screen
		self.image, self.rect = make_bat(self.size, self.color, self.width)
		self.rect.midtop = xy

	def move(self, xy):
		""" Moves the bat to position xy """
		self.rect.midtop = xy[0], xy[1] + self.offset


# ===========================  GHOST  ========================================
class Ghost(pygame.sprite.Sprite):
	""" This class makes it possible to draw the real position of a bat before
		the server has responded with the true position """
	
	def __init__(self, pos, rect, screen, offset):
		""" Inits a ghost object """
		pygame.sprite.Sprite.__init__(self)
		self.screen = screen
		self.offset = offset
		self.image, self.rect = make_ghost(rect)
		self.rect.midtop = pos
	
	def move(self, pos):
		""" Move the ghost to a new position """
		self.rect.midtop = pos[0], pos[1] + self.offset
