#!/usr/bin/env python
# -*- coding: latin-1 -*-
   
""" The player """

# Imports
import sys, pygame, time, os
from pygame.locals import * 
from numarray import array
from balls import *
from bats import *
from pygameutils import *
from networkhandler import *
from prediction import *
from replicators import *
from constants import *

class Player:
	""" Class representing a player in the shuffle puck game """
	PREDICTOR = 0
	PLAYERRECT = 1
	SCREEN = 2
	OFFSET = 3
	COLOR = 4

	WHITE = (255, 255, 255)
	BLACK = (0, 0, 0)
	YELLOW = (255, 255, 0)
	GREEN = (152, 251, 152)
	RED = (255, 99, 71)

	PLAYERPREDICTOR = 0
	OPPONENTPREDICTOR = 1
	BALLPREDICTOR = 2

	POSITION = 0
	LINEAR = 1
	QUADRATIC = 2
	RICHARDSON = 3
	CATMULL = 4

	def __init__(self, listeningPort, playerNumber, serverAddress, serverPort, bg):
		""" Init a player (client) """
		self.debug = 0
		self.playerNumber = playerNumber
		self.receiveBuffer = []
		self.sendBuffer = []
		self.initPygame(bg)
		self.offset = 0
		self.color = (200, 200, 200)
		if playerNumber == 'player2':
			self.offset = 300
			self.color = (100, 100, 200)
		startPos = (550 - 2*(self.offset/(6.0/5.0)), 125)
		self.addBat(startPos)
		self.addBall((400, 150 + self.offset))
		self.network = NetworkClient(self, listeningPort, serverAddress, serverPort)
		self.communictor = ClientCommunicator(self, listeningPort, serverAddress, \
				serverPort)
		self.network.startListening()
		self.communictor.startListening()
		self.batReplicator = BatReplicator(self)
		self.batPredictor = BatPredictor(startPos)
		self.activePredictor = self.batPredictor
		self.predictorState = self.PLAYERPREDICTOR
		self.ballPredictor = BallPredictor()
		self.initPredictorMessage()
		# Framerate stuff
		self.serverFrameRate = 60.0
		self.clientFrameRate = 60.0
		self.connected = 0
		self.batUpdated = 1
		self.lastMoveTime = time.time()
		self.players = {}
		self.fps = 60.0
		self.help = 0
		self.scores = [0, 0]

	def initPygame(self, bg):
		""" Inits pygame engine """
		self.SCREEN_SIZE = (800, 600)
		if not pygame.display.get_init():
			pygame.init()
			self.screen = pygame.display.set_mode(self.SCREEN_SIZE)
			pygame.display.set_caption("Joakim Bechs Master Thesis shuffle puck game")
		else:
			self.screen = pygame.display.get_surface()
		
		# Init mouse
		# Don't show mouse
		pygame.mouse.set_visible(0) 
		# Grab mouse to pygame window
		pygame.event.set_grab(0)
		
		# Set/Show background
		self.background = bg

		# Init clock (for locking frameRate)
		self.clock = pygame.time.Clock()

		# A list of the sprite, i.e bats and ball
		self.sprites = pygame.sprite.RenderClear()
		self.ghosts = pygame.sprite.RenderClear()

		pygame.key.set_repeat(50, 1)
		if pygame.font:
			self.font = pygame.font.Font(pygame.font.match_font("tahoma"), 14)
			self.bigfont = pygame.font.Font(pygame.font.match_font("tahoma"), 24)

	def initPredictorMessage(self):
		""" Initialize messages shown in the game window """
		self.predictorMessage = {}
		self.predictorMessage[self.PLAYERPREDICTOR] = "Position"
		self.predictorMessage[self.OPPONENTPREDICTOR] = "Position"
		self.predictorMessage[self.BALLPREDICTOR] = "Position"
	
	def updateFramerate(self, serverFrameRate):
		""" Method for updating the servers framerate """
		self.serverFrameRate = serverFrameRate
		print 'New serverframerate', self.serverFrameRate

	def getCurrentServerFrameRate(self):
		""" Get the servers current framerate """
		return self.serverFrameRate

	def addBat(self, xy):
		""" Add a bat to the client it self """
		self.bat = ClientBat(xy, self.screen, self.offset, self.color)
		self.sprites.add(self.bat)
		self.ghost = Ghost(xy, self.bat.rect, self.screen, self.offset)
		self.ghosts.add(self.ghost)
		self.drawGhost = 0

	def addBall(self, xy):
		""" Add a ghost ball that represents the ball at server """
		self.ball = ClientBall(xy, self.screen, self.offset)
		self.sprites.add(self.ball)

	def addPlayer(self, addr):
		""" Add a ghost player to the default location """
		self.players[addr] = [BatPredictor((10,10)), ClientBat((10,10), \
				self.screen, self.offset, (100, 100, 100))]
		self.sprites.add(self.players[addr][1])

	def putMessage(self, message):
		""" Put incoming messages into the receivebuffer """
		self.receiveBuffer.append(message)

	def sendMessages(self):
		""" Sends the messages that's in the sendBuffer """
		for message in self.sendBuffer:
			self.network.sendMessage(message)
		self.sendBuffer = []

	def processMessages(self):
		""" Goes thru all incoming messages, one by one. This could be a bad
			idea, that leads to buffer fills up and never get emptied """
		if len(self.receiveBuffer) > 0:
			self.receiveBuffer[0].handleMessage(self)
			self.receiveBuffer = self.receiveBuffer[1:]

	def drawServerFrameRate(self):
		""" Draw the server framerate """
		# Render the status message
		message = "Server FPS: " + str(int(self.serverFrameRate))
		text = self.font.render(message, 1, self.WHITE)
		# Copy the rendered message onto the board
		self.background.fill (self.BLACK, (610, 10, 290, 25))
		self.background.blit (text, (610, 10))

	def drawFPS(self):
		""" Draw the FPS """
		# Render the status message
		startPos = 70
		message = "FPS: " + str(self.fps)
		text = self.font.render(message, 1, self.WHITE)
		# Copy the rendered message onto the board
		self.background.fill (self.BLACK, (610, startPos + self.offset, 290, 25))
		self.background.blit (text, (610, startPos + self.offset))
	
	def drawMiscInfo(self):
		""" Draw misc information about objects """
		startPos = 70
		ghost = "Ghost"
		if self.drawGhost:
			text = self.font.render(ghost, 1, self.GREEN)
		else:
			text = self.font.render(ghost, 1, self.RED)
		self.background.fill (self.BLACK, (610, startPos, 290, 25))
		self.background.blit (text, (610, startPos + self.offset))

	def drawConnected(self):
		""" Show if a client is connected or not """
		if self.connected:
			startPos = 40
			# Render the status message
			player = "1"
			if self.offset > 0:
				player = "2"
			message = "Player " + player + " is connected"
			text = self.font.render(message, 1, self.GREEN)
			# Copy the rendered message onto the board
			sourceRect = (610, startPos + self.offset, 290, 25) 
			self.background.fill (self.BLACK, sourceRect)
			self.background.blit (text, (610, startPos + self.offset))

	def drawScore(self):
		""" Draw the score """
		# Render the status message
		message1 = "P2: " + str(self.scores[1])
		message2 = "P1: " + str(self.scores[0])
		
		xpos1 = 160
		xpos2 = 380
		ypos = 20
		if self.offset > 0:
			xpos = 380
		text1 = self.bigfont.render(message1, 1, self.BLACK)
		text2 = self.bigfont.render(message2, 1, self.BLACK)
		# Copy the rendered message onto the board
		self.background.blit (text1, (xpos1, self.offset + ypos))
		self.background.blit (text2, (xpos2, self.offset + ypos))

	def drawPredictor(self):
		""" Draw what predictor that is used """
		startPos = 100
		separator = 20
		# Render the status message
		color = self.WHITE
		message = "Predictors" 
		text = self.font.render(message, 1, color)
		# Copy the rendered message onto the board
		self.background.fill (self.BLACK, (610, startPos + self.offset, 290, 25))
		self.background.blit (text, (610, startPos + self.offset))

		color = self.WHITE
		if self.predictorState == self.PLAYERPREDICTOR:
			color = self.YELLOW
		message = "Own: " + self.predictorMessage[self.PLAYERPREDICTOR]
		text = self.font.render(message, 1, color)
		# Copy the rendered message onto the board
		self.background.fill (self.BLACK, (610, startPos + separator + \
				self.offset, 290, 25))
		self.background.blit (text, (610, startPos + separator + self.offset))

		color = self.WHITE
		if self.predictorState == self.OPPONENTPREDICTOR:
			color = self.YELLOW
		message = "Opponent: " + self.predictorMessage[self.OPPONENTPREDICTOR]
		text = self.font.render(message, 1, color)
		# Copy the rendered message onto the board
		self.background.fill (self.BLACK, (610, startPos + separator*2 + \
				self.offset, 290, 25))
		self.background.blit (text, (610, startPos + separator*2 + self.offset))

		color = self.WHITE
		if self.predictorState == self.BALLPREDICTOR:
			color = self.YELLOW
		message = "Ball: " + self.predictorMessage[self.BALLPREDICTOR]
		text = self.font.render(message, 1, color)
		# Copy the rendered message onto the board
		self.background.fill (self.BLACK, (610, startPos + separator*3 + \
				self.offset, 290, 25))
		self.background.blit (text, (610, startPos + separator*3 + self.offset))

	def drawPacketPercent(self):
		""" Draw how many percents of the packets that are lost """
		startPos = 200
		# Render the status message
		percentage = int(100 * self.network.dropPercentage)
		message = "Missing packets: " + str(percentage) + " %" 
		color = (255, 255, 255)
		if percentage < 51:
			# Fade from green to white
			max = 5
			fade = (0*percentage + max*percentage)
			color = (fade, 255, fade)
		else:
			# Fade from white to red
			max = 100.0
			min = 51.0
			fade = 255 - ((percentage - min)/(max-min))*255
			color = (255, fade, fade)
			
		text = self.font.render(message, 1, color)
		# Copy the rendered message onto the board
		self.background.fill (self.BLACK, (610, startPos + self.offset, 290, 25))
		self.background.blit (text, (610, startPos + self.offset))

	def changeClientFps(self):
		""" Changes the clients fps """
		self.fps += 1.0
		if self.fps > 70:
			self.fps = 1.0
		print 'Changeing client fps to:', self.fps

	def getFps(self):
		""" Get the clients current fps """
		return self.fps

	def showHelp(self):
		""" Enables and disables the help, this is a dummy function, that
			is needed due to the two player stuff in same application """
		if self.help:
			self.help = 0
			self.background.blit(self.background, (0,0))
		else:
			self.help = 1

	def getHelp(self):
		""" Returns the value of the help variable """
		return self.help

	def renderSprites(self):
		""" Updates the screen """
		if self.drawGhost:
			self.ghosts.clear(self.background, self.background)
			self.ghosts.draw(self.background)
		self.sprites.draw(self.background)

	def move(self, type):
		""" Move the bat """
		self.batUpdated = 0
		if type == LEFT:
			self.batReplicator.setMovement(LEFT)
		elif type == RIGHT:
			self.batReplicator.setMovement(RIGHT)
		elif type == UP:
			self.batReplicator.setMovement(UP)
		elif type == DOWN:
			self.batReplicator.setMovement(DOWN)
		elif type == STOP:
			self.batReplicator.setMovement(STOP)
		self.batUpdated = 1

	def doMouseMovement(self, pos):
		""" Move the bat with the mouse """
		#self.ghost.move(pos)
		self.batReplicator.setPosition(array(pos))

	def updateSpritePositions(self):
		""" Update the position of the sprites/objects """
		newBallPos = self.ballPredictor.getPosition()
		self.ball.move(newBallPos)
		#print 'New ballpos', newBallPos
		newBatPos = self.batPredictor.getPosition() 
		self.bat.move(newBatPos)
		for player in self.players.keys():
			newPos = self.players[player][self.PREDICTOR].getPosition()
			self.players[player][self.PLAYERRECT].move(newPos)
	
	def updateScore(self, message):
		""" Updates the score """
		if message.player == 1:
			self.scores[0] = message.score
		else:
			self.scores[1] = message.score
		self.ballPredictor.setDefaultStartPosition()

	def connect(self, address):
		""" Connects a player to the specific address """
		self.communictor.connect(address)

	def setConnected(self, val):
		""" Sets that a player is connected to the server """
		self.connected = val
		print 'Player got connected'

	def changePredictor(self):
		""" Change prediction method """
		pygame.key.set_repeat(100, 500)
		newState = self.activePredictor.changePredictor()
		if newState == self.POSITION:
			self.predictorMessage[self.predictorState] = "Position"
		if newState == self.LINEAR:
			self.predictorMessage[self.predictorState] = "Linear"
		if newState == self.QUADRATIC:
			self.predictorMessage[self.predictorState] = "Quadratic"
		if newState == self.RICHARDSON:
			self.predictorMessage[self.predictorState] = "Richardson (FIR)"
		if newState == self.CATMULL:
			self.predictorMessage[self.predictorState] = "Catmull-Rom (FIR)"

		pygame.key.set_repeat(50, 1)

	def changeActivePredictor(self):
		""" Change the active predictor """
		pygame.key.set_repeat(100, 500)
		if self.predictorState == self.PLAYERPREDICTOR:
			for player in self.players.keys():
				self.activePredictor = self.players[player][self.PREDICTOR]
				self.predictorState = self.OPPONENTPREDICTOR
				return
			self.predictorState = self.OPPONENTPREDICTOR
		elif self.predictorState == self.OPPONENTPREDICTOR:
			self.activePredictor = self.ballPredictor
			self.predictorState = self.BALLPREDICTOR
		elif self.predictorState == self.BALLPREDICTOR:
			self.activePredictor = self.batPredictor
			self.predictorState = self.PLAYERPREDICTOR
		pygame.key.set_repeat(50, 1)


	def drawText(self):
		""" Draw text """
		self.drawFPS()
		self.drawConnected()
		self.drawPredictor()
		self.drawPacketPercent()
		self.drawScore()

	def beforeHandleInput(self):
		""" Things that should be handled before the windows repaints """
		# Update the sprites positions
		self.updateSpritePositions()

	def pastHandleInput(self):
		""" Things that should be handled after the windows repaints """
		# Update the screen
		self.renderSprites()
