#!/usr/bin/env python
# -*- coding: latin-1 -*-

""" Miscellaneous functions that make thing nice in Pygame """

# Imports
import os, pygame, math
from pygame.locals import *
from numarray import array

def load_image(name, colorkey=None):
	"""Loads an image, convert it and sets a colorkey"""
	fullname = os.path.join('data', name)
	try:
		image = pygame.image.load(fullname)
	except pygame.error, message:
		print 'Cannot load image:', fullname
		raise SystemExit, message
	image = image.convert()
	if colorkey:
		image.set_colorkey(colorkey, RLEACCEL)
	return image

def make_bat(size, color, width=1):
	"""Makes a rectangle with size, color and width.
		This method is a modified version of the one
		available in pythons example uberball"""
	darkcolor = [x+40 for x in color]
	lightcolor = [x-40 for x in color]
	surf = pygame.Surface(size)
	rect = surf.get_rect()
	# Do a rect 1 px smaller
	smallrect = rect.inflate(-width-1, -width-1)
	surf.fill(color)
	# Paint the upper part with a lighter color
	pygame.draw.lines(surf, lightcolor, 0, (smallrect.topright, \
		smallrect.bottomright, smallrect.bottomleft), width)
	# Paint the upper part with a darker color
	pygame.draw.lines(surf, darkcolor, 0, (rect.bottomleft, \
		rect.topleft, rect.topright), width)
	return surf, rect

def make_ghost(irect):
	"""Makes a rectangle with size, color and width.
		This method is a modified version of the one
		available in pythons example uberball"""
	surf = pygame.Surface(irect.size)
	rect = surf.get_rect()
	smallrect = rect.inflate(-2, -2)
	surf.fill((255, 255, 255))
	pygame.draw.lines(surf, (0,0,0), 0, (smallrect.topleft, \
		smallrect.topright, smallrect.bottomright, \
		smallrect.bottomleft, smallrect.topleft), 1)
	surf.set_colorkey(255)
	return surf, rect

def make_ball(radius, color):
	"""Makes a circle with radius and color.
		This method is a modified version of the one
		available in pythons example uberball"""
	darkcolor = [x+50 for x in color]
	size = radius * 2
	surf = pygame.Surface((size, size))
	pygame.draw.circle(surf, color, (radius, radius), radius)
	half = radius/2
	surf.set_at((half, half+1), darkcolor)
	surf.set_at((half+1, half), darkcolor)
	surf.set_colorkey(0)
	return surf, surf.get_rect()

def calculatePosition(angle, speed):
	return array([speed * math.cos(angle), speed * math.sin(angle)])

def calculateAccelerationVectors(angle, acc):
	return array([acc * math.cos(angle), acc * math.sin(angle)])

def calculateDistance(newPos, oldPos):
	""" Calculates the distance between two points """
	return (newPos[0] - oldPos[0]), (newPos[1] - oldPos[1])

def calculateVelocity(newPos, oldPos, dt):
	""" Calculates the velocity depending on its positions and time
		between the positions """
	dx, dy = calculateDistance(newPos, oldPos)
	ds = math.sqrt(dx**2 + dy**2)
	if abs(dt) < 0.0001:
		return -1
	
	return ds/dt

def calculateAcceleration(newSpeed, oldSpeed, dt):
	""" Calculates the accerleration depending on different velocity """
	ns = float(newSpeed)
	os = float(oldSpeed)
	dv = ns - os
	if abs(dt) < 0.0001:
		return -1
	return dv/dt

def calculateAngle(newPos, oldPos):
	""" Calculates the angle between two points """
	enum = newPos[1] - oldPos[1]
	denum = newPos[0] - oldPos[0]
	# Force denumerator to be a low value so we don't divide by zero
	if abs(denum) < 0.0001:
		if enum >= 0:
			return math.pi / 2
		else:
			return -math.pi / 2
	return math.atan2(enum, denum)

def isSamePosition(newPos, oldPos):
	""" Check if it is the same position """
	dx = newPos[0] - oldPos[0]
	dy = newPos[1] - oldPos[1]
	return dx == dy
