#!/usr/bin/env python
# -*- coding: latin-1 -*-

""" The main method for the clients """

# Imports
import sys, pygame, math, time, os
from pygameutils import *
from pygame.locals import * 
from twisted.internet.protocol import DatagramProtocol
from twisted.internet import reactor
from constants import *
import player

""" Inits pygame engine """
screen = None
background = None

SCREEN_SIZE = (800, 600)
if not pygame.display.get_init():
	pygame.init()
	screen = pygame.display.set_mode(SCREEN_SIZE)
	pygame.display.set_caption("Joakim Bechs demo application")
else:
	self.screen = pygame.display.get_surface()

pygame.key.set_repeat()

background = load_image('background.jpg').convert()
bgpic = load_image('background.jpg').convert()
showHelp = 0

def handleInput(p1, p2):
	""" Handles input from the client """
	global showHelp
	for event in pygame.event.get():
		if event.type == QUIT:
			sys.exit() # Quit the game
		elif event.type == KEYDOWN:
			if event.key == K_ESCAPE:
				sys.exit() # Quit the game

			# Player 1 movements
			elif event.key == K_LEFT:
				p1.move(LEFT)
			elif event.key ==  K_RIGHT:
				p1.move(RIGHT)
			elif event.key == K_UP:
				p1.move(UP)
			elif event.key == K_DOWN:
				p1.move(DOWN)
			# Player 2 movements
			elif event.key == K_a:
				p2.move(LEFT)
			elif event.key ==  K_d:
				p2.move(RIGHT)
			elif event.key == K_w:
				p2.move(UP)
			elif event.key == K_s:
				p2.move(DOWN)

			# Connect keys
			elif event.key == K_z:
				p1.connect((p1.network.ip, p1.network.port))
			elif event.key == K_x:
				p2.connect((p2.network.ip, p2.network.port))
			
			# Settings keys
			elif event.key == K_q:
				p1.changeActivePredictor()
			elif event.key == K_1:
				p1.changePredictor()
			elif event.key == K_2:
				p1.network.subDropPercent()
			elif event.key == K_3:
				p1.network.addDropPercent()
			elif event.key == K_e:
				p1.changeClientFps()
				
			elif event.key == K_u:
				p2.changeActivePredictor()
			elif event.key == K_7:
				p2.changePredictor()
			elif event.key == K_8:
				p2.network.subDropPercent()
			elif event.key == K_9:
				p2.network.addDropPercent()
			elif event.key == K_o:
				p2.changeClientFps()

		elif event.type == KEYUP:
			# Send to messages from player1
			if event.key == K_UP or event.key == K_DOWN or \
					event.key == K_LEFT or event.key == K_RIGHT:
				p1.move(STOP)

			# Send to messages from player2
			if event.key == K_w or event.key == K_s or \
					event.key == K_a or event.key == K_d:
				p2.move(STOP)

			# Commands to server
			elif event.key == K_r:
				newServerFrameRate = p1.getCurrentServerFrameRate() + 10
				if newServerFrameRate > 100:
					newServerFrameRate = 10
				p1.communictor.changeServerFramerate(newServerFrameRate)
			elif event.key == K_g:
				p1.drawGhost = not p1.drawGhost
				p2.drawGhost = not p2.drawGhost

			# Help command
			elif event.key == K_h:
				print 'Enable/disable help'
				showHelp = not showHelp

	# Player one can use mouse also
	if pygame.mouse.get_rel() != (0,0):
		p1.doMouseMovement(pygame.mouse.get_pos())
	
def drawServerFrameRate(font, player):
	""" Draw the servers framerate """
	# Render the status message
	message = "Server FPS: " + str(int(player.serverFrameRate))
	text = font.render(message, 1, (255, 255, 255))
	# Copy the rendered message onto the board
	background.fill ((0,0,0), (610, 10, 290, 25))
	background.blit (text, (610, 10))

def drawHelp(font, smallfont):
	""" Draw the help dialog """
	# Render the help messages
	startPos = 30
	offset = 50
	smallOffset = 12
	background.fill ((0,0,0), (100, 100, 400, 400))
	message = "Player 1"
	text = font.render(message, 1, GREEN)
	background.blit (text, (120, startPos + offset + 30))
	message = "Movement"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120, startPos + offset*2))
	message = "Change FPS"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120, startPos + offset*3))
	message = "Change packet drop"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120, startPos + offset*4))
	message = "Connection"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120, startPos + offset*5))
	message = "Prediction"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120, startPos + offset*6))

	message = "left, right, up and down"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130, startPos + offset*2 + smallOffset + 6))
	message = "Mouse movement"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130, startPos + offset*2 + smallOffset*2 + 6))
	message = "e increments fps"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130, startPos + offset*3 + smallOffset + 6))
	message = "2 decrements drop"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130, startPos + offset*4 + smallOffset + 6))
	message = "3 increments drop"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130, startPos + offset*4 + smallOffset*2 + 6))
	message = "z connnects to server"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130, startPos + offset*5 + smallOffset + 6))
	message = "q change object"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130, startPos + offset*6 + smallOffset + 6))
	message = "1 next prediction method"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130, startPos + offset*6 + smallOffset*2 + 6))


	player2offset = 200
	message = "Player 2"
	text = font.render(message, 1, GREEN)
	background.blit (text, (120 + player2offset, startPos + offset + 30))
	message = "Movement"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120 + player2offset, startPos + offset*2))
	message = "Change FPS"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120 + player2offset, startPos + offset*3))
	message = "Change packet drop"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120 + player2offset, startPos + offset*4))
	message = "Connection"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120 + player2offset, startPos + offset*5))
	message = "Prediction"
	text = font.render(message, 1, YELLOW)
	background.blit (text, (120 + player2offset, startPos + offset*6))

	message = "a, d, w, s"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130 + player2offset, startPos + offset*2 + smallOffset + 6))
	message = "o increments fps"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130 + player2offset, startPos + offset*3 + smallOffset + 6))
	message = "8 decrements drop"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130 + player2offset, startPos + offset*4 + smallOffset + 6))
	message = "9 increments drop"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130 + player2offset, startPos + offset*4 + smallOffset*2 + 6))
	message = "x connnects to server"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130 + player2offset, startPos + offset*5 + smallOffset + 6))
	message = "u change object"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130 + player2offset, startPos + offset*6 + smallOffset + 6))
	message = "7 next prediction method"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130 + player2offset, startPos + offset*6 + smallOffset*2 + 6))

	message = "Server commands"
	text = font.render(message, 1, GREEN)
	background.blit (text, (120, startPos + offset*7 + 30))
	message = "r change the servers framerate"
	text = smallfont.render(message, 1, WHITE)
	background.blit (text, (130 , startPos + offset*7 + 30 + smallOffset + 6))

def main():
	serverAddress = '127.0.0.1'
	serverPort = 5555
	playerOne = player.Player(6666, 'player1', \
			serverAddress, serverPort, background)
	playerTwo = player.Player(7777, 'player2', \
			serverAddress, serverPort, background)
	running = 1
	# Framerate handling
	clock = pygame.time.Clock()
	lastTimeP1 = lastTimeP2 = time.time()

	# Font stuff that is for the application and not players
	if pygame.font:
		font = pygame.font.Font(pygame.font.match_font("tahoma"), 16)
		smallfont = pygame.font.Font(pygame.font.match_font("tahoma"), 12)
	else:
		print 'Ingen font'
		font = None
	help = 1

	while running:
		background.blit(bgpic, (0,0))

		timeNow = time.time()
		timeDiffP1 = timeNow - lastTimeP1
		timeDiffP2 = timeNow - lastTimeP2
		renderP1 = 0
		renderP2 = 0
		# To be able to have to different framerates in the same client we have
		# to check this every "real" frame
		if timeDiffP1 > (1.0 / playerOne.getFps()):
			lastTimeP1 = timeNow
			renderP1 = 1
		if timeDiffP2 > (1.0 / playerTwo.getFps()):
			lastTimeP2 = timeNow
			renderP2 = 1

		if renderP1:
			playerOne.beforeHandleInput()

		if renderP2:
			playerTwo.beforeHandleInput()
	
		# Need to do input handling from players outside the player objects,
		# this due to an issue with event queue in pygame
		if renderP1 or renderP2:
			handleInput(playerOne, playerTwo)



		drawServerFrameRate(font, playerOne)
		playerOne.drawText() 
		playerTwo.drawText()
		playerOne.renderSprites()
		playerTwo.renderSprites()
		if showHelp:
			drawHelp(font, smallfont)
		screen.blit(background, (0, 0))
		pygame.display.flip()
		pygame.event.pump()
		reactor.doSelect(0)
		reactor.runUntilCurrent()

if __name__ == '__main__':
	main()
